﻿// Copyright (c) Microsoft Open Technologies, Inc. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using System;
using Microsoft.Data.Entity.Tests;
using Microsoft.Data.FunctionalTests;
using Microsoft.Framework.DependencyInjection;
using Microsoft.Framework.DependencyInjection.Advanced;
using Microsoft.Framework.DependencyInjection.Fallback;
using Xunit;

namespace Microsoft.Data.Entity.SQLite.FunctionalTests
{
    public class NorthwindQueryTest : NorthwindQueryTestBase, IClassFixture<NorthwindQueryFixture>
    {
        public override void Take_with_single()
        {
            base.Take_with_single();

            Assert.Equal(
                @"SELECT ""Address"", ""City"", ""CompanyName"", ""ContactName"", ""ContactTitle"", ""Country"", ""CustomerID"", ""Fax"", ""Phone"", ""PostalCode"", ""Region""
FROM ""Customers""
ORDER BY ""CustomerID""
LIMIT 1",
                _fixture.Sql);
        }

        public override void String_StartsWith_Literal()
        {
            base.String_StartsWith_Literal();

            Assert.Equal(
                @"SELECT ""Address"", ""City"", ""CompanyName"", ""ContactName"", ""ContactTitle"", ""Country"", ""CustomerID"", ""Fax"", ""Phone"", ""PostalCode"", ""Region""
FROM ""Customers""
WHERE ""ContactName"" LIKE @p0 || '%'",
                _fixture.Sql);
        }

        public override void String_StartsWith_MethodCall()
        {
            base.String_StartsWith_MethodCall();

            Assert.Equal(
                @"SELECT ""Address"", ""City"", ""CompanyName"", ""ContactName"", ""ContactTitle"", ""Country"", ""CustomerID"", ""Fax"", ""Phone"", ""PostalCode"", ""Region""
FROM ""Customers""
WHERE ""ContactName"" LIKE @p0 || '%'",
                _fixture.Sql);
        }

        public override void String_EndsWith_Literal()
        {
            base.String_EndsWith_Literal();

            Assert.Equal(
                @"SELECT ""Address"", ""City"", ""CompanyName"", ""ContactName"", ""ContactTitle"", ""Country"", ""CustomerID"", ""Fax"", ""Phone"", ""PostalCode"", ""Region""
FROM ""Customers""
WHERE ""ContactName"" LIKE '%' || @p0",
                _fixture.Sql);
        }

        private readonly NorthwindQueryFixture _fixture;

        public NorthwindQueryTest(NorthwindQueryFixture fixture)
        {
            _fixture = fixture;
            _fixture.InitLogger();
        }

        protected override DbContext CreateContext()
        {
            return _fixture.CreateContext();
        }
    }

    public class NorthwindQueryFixture : NorthwindQueryFixtureBase, IDisposable
    {
        private readonly TestSqlLoggerFactory _loggingFactory = new TestSqlLoggerFactory();

        private readonly IServiceProvider _serviceProvider;
        private readonly DbContextOptions _options;
        private readonly TestDatabase _testDatabase;

        public NorthwindQueryFixture()
        {
            _serviceProvider
                = new ServiceCollection()
                    .AddEntityFramework()
                    .AddSQLite()
                    .UseLoggerFactory(_loggingFactory)
                    .ServiceCollection
                    .BuildServiceProvider();

            _testDatabase = TestDatabase.Northwind();

            _options = new DbContextOptions()
                .UseModel(CreateModel())
                .UseSQLite(_testDatabase.Connection.ConnectionString);
        }

        public DbContext CreateContext()
        {
            return new DbContext(_serviceProvider, _options);
        }

        public string Sql
        {
            get { return string.Join("\r\n\r\n", TestSqlLoggerFactory.Logger._sqlStatements); }
        }

        public void Dispose()
        {
            _testDatabase.Dispose();
        }

        public void InitLogger()
        {
            _loggingFactory.Init();
        }
    }
}
